@@ -633,6 +633,8 @@ static NvBool nv_numa_node_has_memory(int node_id)
         free_pages(ptr, order);                      \
     }
 
+extern NvU64 nv_shared_gpa_boundary;
+
 static inline pgprot_t nv_adjust_pgprot(pgprot_t vm_prot, NvU32 extra)
 {
     pgprot_t prot = __pgprot(pgprot_val(vm_prot) | extra);
@@ -644,7 +646,18 @@ static inline pgprot_t nv_adjust_pgprot(pgprot_t vm_prot, NvU32 extra)
      * If cc_mkdec() is present, then pgprot_decrypted() can't be used.
      */
 #if defined(NV_CC_MKDEC_PRESENT)
-    prot =  __pgprot(__sme_clr(pgprot_val(vm_prot)));
+    if (nv_shared_gpa_boundary != 0)
+    {
+        /*
+         * By design, a VM using vTOM doesn't see the SEV setting and
+         * for AMD with vTOM, *set* means decrypted.
+         */
+        prot =  __pgprot(nv_shared_gpa_boundary | (pgprot_val(vm_prot)));
+    }
+    else
+    {
+        prot =  __pgprot(__sme_clr(pgprot_val(vm_prot)));
+    }
 #else
     prot = pgprot_decrypted(prot);
 #endif
@@ -1438,6 +1451,17 @@ struct nv_dma_device {
     NvBool nvlink;
 };
 
+/* Properties of the coherent link */
+typedef struct coherent_link_info_s {
+    /* Physical Address of the GPU memory in SOC AMAP. In the case of
+     * baremetal OS environment it is System Physical Address(SPA) and in the case
+     * of virutalized OS environment it is Intermediate Physical Address(IPA) */
+    NvU64 gpu_mem_pa;
+    /* Bitmap of NUMA node ids, corresponding to the reserved PXMs,
+     * available for adding GPU memory to the kernel as system RAM */
+    DECLARE_BITMAP(free_node_bitmap, MAX_NUMNODES);
+} coherent_link_info_t;
+
 #if defined(NV_LINUX_ACPI_EVENTS_SUPPORTED)
 /*
  * acpi data storage structure
@@ -1471,6 +1495,13 @@ typedef struct nv_linux_state_s {
     /* IBM-NPU info associated with this GPU */
     nv_ibmnpu_info_t *npu;
 
+    /* coherent link information */
+     coherent_link_info_t coherent_link_info;
+
+    /* Dedicated queue to be used for removing FB memory which is onlined
+     * to kernel as a NUMA node. Refer Bug : 3879845*/
+    nv_kthread_q_t remove_numa_memory_q;
+
     /* NUMA node information for the platforms where GPU memory is presented
      * as a NUMA node to the kernel */
     struct {
@@ -1481,6 +1512,7 @@ typedef struct nv_linux_state_s {
         /* NUMA online/offline status for platforms that support GPU memory as
          * NUMA node */
         atomic_t status;
+        NvBool use_auto_online;
     } numa_info;
 
     nvidia_stack_t *sp[NV_DEV_STACK_COUNT];
@@ -1946,6 +1978,36 @@ static inline int nv_set_numa_status(nv_linux_state_t *nvl, int status)
     return 0;
 }
 
+static inline NvBool nv_platform_use_auto_online(nv_linux_state_t *nvl)
+{
+    return nvl->numa_info.use_auto_online;
+}
+
+typedef struct {
+    NvU64 base;
+    NvU64 size;
+    NvU32 nodeId;
+    int ret;
+} remove_numa_memory_info_t;
+
+static void offline_numa_memory_callback
+(
+    void *args
+)
+{
+#ifdef NV_OFFLINE_AND_REMOVE_MEMORY_PRESENT
+    remove_numa_memory_info_t *pNumaInfo = (remove_numa_memory_info_t *)args;
+#ifdef NV_REMOVE_MEMORY_HAS_NID_ARG
+    pNumaInfo->ret = offline_and_remove_memory(pNumaInfo->nodeId,
+                                               pNumaInfo->base,
+                                               pNumaInfo->size);
+#else
+    pNumaInfo->ret = offline_and_remove_memory(pNumaInfo->base,
+                                               pNumaInfo->size);
+#endif
+#endif
+}
+
 typedef enum
 {
     NV_NUMA_STATUS_DISABLED             = 0,
