@@ -24,6 +24,21 @@
 #ifndef _MINION_NVLINK_DEFINES_PUBLIC_H_
 #define _MINION_NVLINK_DEFINES_PUBLIC_H_
 
+//PAD REG READ API (Bug 2643883)
+#define NV_MINION_UCODE_READUPHYPAD_ADDR 11:0
+#define NV_MINION_UCODE_READUPHYPAD_LANE 15:12
+
+//FIELD FOR DEBUG_MISC_i DATA REGISTERS
+#define NV_MINION_DEBUG_MISC_0_LINK_STATE 7:0
+#define NV_MINION_DEBUG_MISC_0_ISR_ID 15:8
+#define NV_MINION_DEBUG_MISC_0_OTHER_DATA 31:16
+
+// Recal values checks
+#define NV_MINION_UCODE_L1_EXIT_MARGIN 100
+#define NV_MINION_UCODE_L1_EXIT_MAX    200
+#define NV_MINION_UCODE_RECOVERY_TIME  250
+#define NV_MINION_UCODE_PEQ_TIME       96
+
 // SUBCODES for DLCMD FAULT (uses DLCMDFAULR code)  - dlCmdFault() - NVLINK_LINK_INT 
 typedef enum _MINION_STATUS
   {
@@ -31,4 +46,6 @@ typedef enum _MINION_STATUS
     MINION_ALARM_BUSY                                   = 80,
 } MINION_STATUS;
 
+  #define LINKSTATUS_EMERGENCY_SHUTDOWN         0x29
+  #define LINKSTATUS_INITPHASE1                 0x24
 #endif // _MINION_NVLINK_DEFINES_PUBLIC_H_
