@@ -202,7 +202,8 @@ extern "C" {
 #define NV2080_NOTIFIERS_NVLINK_INFO_LINK_DOWN                     (176)
 #define NV2080_NOTIFIERS_NVPCF_EVENTS                              (177)
 #define NV2080_NOTIFIERS_HDMI_FRL_RETRAINING_REQUEST               (178)
-#define NV2080_NOTIFIERS_MAXCOUNT                                  (179)
+#define NV2080_NOTIFIERS_VRR_SET_TIMEOUT                           (179)
+#define NV2080_NOTIFIERS_MAXCOUNT                                  (180)
 
 // Indexed GR notifier reference
 #define NV2080_NOTIFIERS_GR(x)         ((x == 0) ? (NV2080_NOTIFIERS_GR0) : (NV2080_NOTIFIERS_GR1 + (x - 1)))
@@ -497,6 +498,10 @@ typedef struct {
     NvU32 linkId;
 } Nv2080NvlinkLnkChangeNotification;
 
+typedef struct {
+    NvU32 head;
+} Nv2080VrrSetTimeoutNotification;
+
 #ifdef __cplusplus
 };     /* extern "C" */
 #endif
