@@ -1837,21 +1837,16 @@ static void HsConfigInitFlipQueue(
 }
 
 static void HsConfigUpdateSurfaceRefCount(
-    NVDevEvoPtr pDevEvo,
     const NVHsChannelConfig *pChannelConfig,
     NvBool increase)
 {
-    HsChangeSurfaceFlipRefCount(
-        pDevEvo, pChannelConfig->warpMesh.pSurface, increase);
+    HsChangeSurfaceFlipRefCount(pChannelConfig->warpMesh.pSurface, increase);
 
-    HsChangeSurfaceFlipRefCount(
-        pDevEvo, pChannelConfig->pBlendTexSurface, increase);
+    HsChangeSurfaceFlipRefCount(pChannelConfig->pBlendTexSurface, increase);
 
-    HsChangeSurfaceFlipRefCount(
-        pDevEvo, pChannelConfig->pOffsetTexSurface, increase);
+    HsChangeSurfaceFlipRefCount(pChannelConfig->pOffsetTexSurface, increase);
 
-    HsChangeSurfaceFlipRefCount(
-        pDevEvo, pChannelConfig->cursor.pSurfaceEvo, increase);
+    HsChangeSurfaceFlipRefCount(pChannelConfig->cursor.pSurfaceEvo, increase);
 }
 
 /*!
@@ -2263,7 +2258,6 @@ void nvHsConfigStart(
              */
             if (pHsConfigOneHead->pHsChannel != NULL) {
                 HsConfigUpdateSurfaceRefCount(
-                    pDevEvo,
                     &pHsConfigOneHead->channelConfig,
                     TRUE /* increase */);
             }
@@ -2274,7 +2268,6 @@ void nvHsConfigStart(
              */
             if (pDispEvo->pHsChannel[apiHead] != NULL) {
                 HsConfigUpdateSurfaceRefCount(
-                    pDevEvo,
                     &pDispEvo->pHsChannel[apiHead]->config,
                     FALSE /* increase */);
             }
