@@ -152,7 +152,7 @@ typedef enum
     /**
      * No valid aligned 4K PTE state for a 64K PTE
      * 64K big PTE state indicating that there is no valid 4K aligned PTEs
-     * 
+     *
      * @note This is not supported pre Volta.
      */
     MMU_WALK_FILL_NV4K,
@@ -376,7 +376,7 @@ MmuWalkCBCopyEntries
  *                           copied.
  * @param[in] tableSize      Size of the current level of PD/PT, in
  *                           entries. The offsets into the staging
- *                           buffer are the entry indices taken 
+ *                           buffer are the entry indices taken
  *                           modulo tableSize.
  * @param[in] entrySize      Size of each entry, in bytes
  */
@@ -509,7 +509,7 @@ struct MMU_MAP_TARGET
      * Callback to map the batch of entries.
      */
     MmuWalkCBMapNextEntries *MapNextEntries;
-    
+
     /*!
      * Page array granularity of the physical target memory
      */
@@ -669,7 +669,7 @@ mmuWalkReleaseEntries
  *
  * Traverse the walker and rewrite the PDEs from the SW state.
  * This won't trigger any new PDE allocations or state change.
- * 
+ *
  * This call won't affect the PTEs. If needed, support can be added later.
  *
  * The VA range must be aligned to the MMU's smallest page size and
@@ -750,11 +750,11 @@ mmuWalkGetPageLevelInfo
     NvU32                   *pMemSize
 );
 
-/*! 
+/*!
  * Force frees all page level instances. We may have to force free page levels
- * in case of surprise removal. In the surprise removal case, we may end up 
+ * in case of surprise removal. In the surprise removal case, we may end up
  * with many failed unmappings once the GPU is off the bus. This might leave
- * some of the MMU_WALK_LEVEL and MMU_WALK_LEVEL_INST objects to be in an 
+ * some of the MMU_WALK_LEVEL and MMU_WALK_LEVEL_INST objects to be in an
  * allocated state. This function just iterates over the level instances at
  * each level and force frees everything ignoring any outstanding valid, sparse
  * and reserved entries..
@@ -834,7 +834,7 @@ mmuWalkGetUserCtx
 /*!
  * Set the user context of a walker state.
  */
-NV_STATUS
+void
 mmuWalkSetUserCtx
 (
     MMU_WALK          *pWalk,
