@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -27,6 +27,8 @@
 #include "gpu/mem_mgr/virt_mem_allocator_common.h"
 #include "os/nv_memory_type.h"
 #include "core/locks.h"
+#include "ctrl/ctrl2080.h"
+#include "rmapi/rs_utils.h"
 
 #include "gpu/bus/kern_bus.h"
 
@@ -42,10 +44,364 @@
 static TRANSFER_TYPE
 memmgrGetMemTransferType
 (
-    MemoryManager *pMemoryManager
+    MemoryManager    *pMemoryManager,
+    TRANSFER_SURFACE *pDst,
+    TRANSFER_SURFACE *pSrc
 )
 {
-    return TRANSFER_TYPE_PROCESSOR;
+    TRANSFER_TYPE transferType        = TRANSFER_TYPE_PROCESSOR;
+    OBJGPU    *pGpu       = ENG_GET_GPU(pMemoryManager);
+    KernelBus *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
+
+    //
+    // In case of copy, both dest and src will be passed
+    // In case of memset/memread/memwrite either dest or src will be passed
+    //
+    if ((pDst != NULL) && (pSrc != NULL) &&
+        (memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM) &&
+        (memdescGetAddressSpace(pSrc->pMemDesc) == ADDR_SYSMEM))
+    {
+        transferType = TRANSFER_TYPE_PROCESSOR;
+    }
+    else if (((pDst != NULL) &&
+             (memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM)) ||
+             ((pSrc != NULL) &&
+             (memdescGetAddressSpace(pSrc->pMemDesc) == ADDR_SYSMEM)))
+    {
+        transferType = TRANSFER_TYPE_PROCESSOR;
+    }
+    else if (kbusIsBarAccessBlocked(pKernelBus))
+    {
+        transferType = TRANSFER_TYPE_GSP_DMA;
+    }
+    return transferType;
+}
+
+static NV_STATUS
+_memmgrAllocAndMapSurface
+(
+    OBJGPU             *pGpu,
+    NvU64               size,
+    MEMORY_DESCRIPTOR **ppMemDesc,
+    void              **ppMap,
+    void              **ppPriv
+)
+{
+    NV_STATUS status;
+    NvU64 flags = 0;
+
+    NV_ASSERT_OR_RETURN(ppMemDesc != NULL, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(ppMap != NULL, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(ppPriv != NULL, NV_ERR_INVALID_ARGUMENT);
+
+    flags = MEMDESC_FLAGS_ALLOC_IN_UNPROTECTED_MEMORY;
+
+    NV_ASSERT_OK_OR_RETURN(
+        memdescCreate(ppMemDesc, pGpu, size, RM_PAGE_SIZE, NV_TRUE,
+                      ADDR_SYSMEM, NV_MEMORY_UNCACHED, flags));
+
+    NV_ASSERT_OK_OR_GOTO(status, memdescAlloc(*ppMemDesc), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status,
+        memdescMapOld(*ppMemDesc, 0, size, NV_TRUE, NV_PROTECT_READ_WRITE,
+                      ppMap, ppPriv),
+        failed);
+
+    // Clear surface before use
+    portMemSet(*ppMap, 0, size);
+
+    return NV_OK;
+failed:
+    memdescFree(*ppMemDesc);
+    memdescDestroy(*ppMemDesc);
+
+    *ppMemDesc = NULL;
+    *ppMap = NULL;
+    *ppPriv = NULL;
+
+    return status;
+}
+
+static void
+_memmgrUnmapAndFreeSurface
+(
+    MEMORY_DESCRIPTOR *pMemDesc,
+    void              *pMap,
+    void              *pPriv
+)
+{
+    memdescUnmapOld(pMemDesc, NV_TRUE, 0, pMap, pPriv);
+
+    memdescFree(pMemDesc);
+    memdescDestroy(pMemDesc);
+}
+
+/*!
+ * @brief This function is used for writing/reading data to/from a client
+ *        provided buffer from/to some source region in vidmem
+ *
+ * @param[in] pDst    TRANSFER_SURFACE info for destination region
+ * @param[in] pBuf    Client provided buffer
+ * @param[in] size    Size in bytes of the memory transfer
+ * @param[in] bRead   TRUE for read and FALSE for write
+ */
+static NV_STATUS
+_memmgrMemReadOrWriteWithGsp
+(
+    OBJGPU           *pGpu,
+    TRANSFER_SURFACE *pDst,
+    void             *pBuf,
+    NvU64             size,
+    NvBool            bRead
+)
+{
+    NV2080_CTRL_INTERNAL_MEMMGR_MEMORY_TRANSFER_WITH_GSP_PARAMS gspParams;
+    NV_STATUS status;
+    MEMORY_DESCRIPTOR *pStagingBuf = NULL;
+    void *pStagingBufMap = NULL;
+    void *pStagingBufPriv = NULL;
+    RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
+
+    // Do not expect GSP to be used for reading/writing from/to sysmem
+    if (memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM)
+        return NV_ERR_NOT_SUPPORTED;
+
+    // Allocate and map the staging buffer
+    NV_ASSERT_OK_OR_RETURN(
+        _memmgrAllocAndMapSurface(pGpu, size, &pStagingBuf, &pStagingBufMap,
+                                  &pStagingBufPriv));
+
+    // Copy the data to staging buffer before poking GSP for copying
+    if (!bRead)
+        portMemCopy(pStagingBufMap, size, pBuf, size);
+
+    // Setup control call params
+    portMemSet(&gspParams, 0, sizeof(gspParams));
+
+    gspParams.memop = NV2080_CTRL_MEMMGR_MEMORY_OP_MEMCPY;
+    gspParams.transferSize = size;
+
+    if (bRead)
+    {
+        // Source surface in vidmem
+        gspParams.src.baseAddr = memdescGetPhysAddr(pDst->pMemDesc, AT_GPU, 0);
+        gspParams.src.size = memdescGetSize(pDst->pMemDesc);
+        gspParams.src.offset = pDst->offset;
+        gspParams.src.cpuCacheAttrib = memdescGetCpuCacheAttrib(pDst->pMemDesc);
+        gspParams.src.aperture = memdescGetAddressSpace(pDst->pMemDesc);
+
+        // Destination surface in unprotected sysmem
+        gspParams.dst.baseAddr = memdescGetPhysAddr(pStagingBuf, AT_GPU, 0);
+        gspParams.dst.size = memdescGetSize(pStagingBuf);
+        gspParams.dst.offset = 0;
+        gspParams.dst.cpuCacheAttrib = memdescGetCpuCacheAttrib(pStagingBuf);
+        gspParams.dst.aperture = memdescGetAddressSpace(pStagingBuf);
+    }
+    else
+    {
+        // Source surface in unprotected sysmem
+        gspParams.src.baseAddr = memdescGetPhysAddr(pStagingBuf, AT_GPU, 0);
+        gspParams.src.size = memdescGetSize(pStagingBuf);
+        gspParams.src.offset = 0;
+        gspParams.src.cpuCacheAttrib = memdescGetCpuCacheAttrib(pStagingBuf);
+        gspParams.src.aperture = memdescGetAddressSpace(pStagingBuf);
+
+        // Destination surface in vidmem
+        gspParams.dst.baseAddr = memdescGetPhysAddr(pDst->pMemDesc, AT_GPU, 0);
+        gspParams.dst.size = memdescGetSize(pDst->pMemDesc);
+        gspParams.dst.offset = pDst->offset;
+        gspParams.dst.cpuCacheAttrib = memdescGetCpuCacheAttrib(pDst->pMemDesc);
+        gspParams.dst.aperture = memdescGetAddressSpace(pDst->pMemDesc);
+    }
+
+    // Send the control call
+    NV_ASSERT_OK_OR_GOTO(status,
+        pRmApi->Control(pRmApi,
+                        pGpu->hInternalClient,
+                        pGpu->hInternalSubdevice,
+                        NV2080_CTRL_CMD_INTERNAL_MEMMGR_MEMORY_TRANSFER_WITH_GSP,
+                        &gspParams,
+                        sizeof(gspParams)),
+        failed);
+
+    // Read contents from staging buffer after GSP is done copying
+    if (bRead)
+        portMemCopy(pBuf, size, pStagingBufMap, size);
+
+failed:
+    _memmgrUnmapAndFreeSurface(pStagingBuf, pStagingBufMap, pStagingBufPriv);
+    return status;
+}
+
+/*!
+ * @brief This function is used for copying data b/w two memory regions
+ *        using GSP.
+ *
+ * @param[in] pDst    TRANSFER_SURFACE info for destination region
+ * @param[in] pSrc    TRANSFER_SURFACE info for source region
+ * @param[in] size    Size in bytes of the memory transfer
+ */
+static NV_STATUS
+_memmgrMemcpyWithGsp
+(
+    OBJGPU           *pGpu,
+    TRANSFER_SURFACE *pDst,
+    TRANSFER_SURFACE *pSrc,
+    NvU64             size
+)
+{
+    NV2080_CTRL_INTERNAL_MEMMGR_MEMORY_TRANSFER_WITH_GSP_PARAMS gspParams;
+    NV_STATUS status;
+    MEMORY_DESCRIPTOR *pStagingBuf = NULL;
+    void *pStagingBufMap = NULL;
+    void *pStagingBufPriv = NULL;
+    NvU8 *pMap = NULL;
+    void *pPriv = NULL;
+    RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
+
+    //
+    // Do not expect GSP to be used for copying data b/w two surfaces
+    // in sysmem. For SPT, there is no non-CPR vidmem. So, allow vidmem
+    // to vidmem copies in plain text. For copies b/w CPR and non-CPR
+    // vidmem, encryption/decryption needs to happen at the endpoints.
+    //
+    if (memdescGetAddressSpace(pSrc->pMemDesc) == ADDR_SYSMEM &&
+        memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM)
+    {
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    // Allocate and map the bounce buffer
+    NV_ASSERT_OK_OR_RETURN(
+        _memmgrAllocAndMapSurface(pGpu, size, &pStagingBuf, &pStagingBufMap,
+                                  &pStagingBufPriv));
+
+    // Setup control call params
+    portMemSet(&gspParams, 0, sizeof(gspParams));
+
+    gspParams.memop = NV2080_CTRL_MEMMGR_MEMORY_OP_MEMCPY;
+    gspParams.transferSize = size;
+
+    if (memdescGetAddressSpace(pSrc->pMemDesc) == ADDR_SYSMEM)
+    {
+        NV_ASSERT_OK_OR_GOTO(status,
+            memdescMapOld(pSrc->pMemDesc, 0, size, NV_TRUE,
+                          NV_PROTECT_READ_WRITE, (void**)&pMap, &pPriv),
+            failed);
+
+        // Copy to staging buffer
+        portMemCopy(pStagingBufMap, size, pMap + pSrc->offset, size);
+
+        memdescUnmapOld(pSrc->pMemDesc, NV_TRUE, 0, (void*)pMap, pPriv);
+
+        // Source surface in unprotected sysmem
+        gspParams.src.baseAddr = memdescGetPhysAddr(pStagingBuf, AT_GPU, 0);
+        gspParams.src.size = memdescGetSize(pStagingBuf);
+        gspParams.src.offset = 0;
+        gspParams.src.cpuCacheAttrib = memdescGetCpuCacheAttrib(pStagingBuf);
+        gspParams.src.aperture = memdescGetAddressSpace(pStagingBuf);
+
+        // Destination surface in vidmem
+        gspParams.dst.baseAddr = memdescGetPhysAddr(pDst->pMemDesc, AT_GPU, 0);
+        gspParams.dst.size = memdescGetSize(pDst->pMemDesc);
+        gspParams.dst.offset = pDst->offset;
+        gspParams.dst.cpuCacheAttrib = memdescGetCpuCacheAttrib(pDst->pMemDesc);
+        gspParams.dst.aperture = memdescGetAddressSpace(pDst->pMemDesc);
+    }
+    else
+    {
+        // Source surface in vidmem
+        gspParams.src.baseAddr = memdescGetPhysAddr(pSrc->pMemDesc, AT_GPU, 0);
+        gspParams.src.size = memdescGetSize(pSrc->pMemDesc);
+        gspParams.src.offset = pSrc->offset;
+        gspParams.src.cpuCacheAttrib = memdescGetCpuCacheAttrib(pSrc->pMemDesc);
+        gspParams.src.aperture = memdescGetAddressSpace(pSrc->pMemDesc);
+
+        if (memdescGetAddressSpace(pDst->pMemDesc) == ADDR_FBMEM)
+        {
+            // Destination surface in vidmem
+            gspParams.dst.baseAddr = memdescGetPhysAddr(pDst->pMemDesc, AT_GPU, 0);
+            gspParams.dst.size = memdescGetSize(pDst->pMemDesc);
+            gspParams.dst.offset = pDst->offset;
+            gspParams.dst.cpuCacheAttrib = memdescGetCpuCacheAttrib(pDst->pMemDesc);
+            gspParams.dst.aperture = memdescGetAddressSpace(pDst->pMemDesc);
+        }
+        else
+        {
+            // Destination surface in unprotected sysmem
+            gspParams.dst.baseAddr = memdescGetPhysAddr(pStagingBuf, AT_GPU, 0);
+            gspParams.dst.size = memdescGetSize(pStagingBuf);
+            gspParams.dst.offset = 0;
+            gspParams.dst.cpuCacheAttrib = memdescGetCpuCacheAttrib(pStagingBuf);
+            gspParams.dst.aperture = memdescGetAddressSpace(pStagingBuf);
+        }
+    }
+
+    // Send the control call
+    NV_ASSERT_OK_OR_GOTO(status,
+        pRmApi->Control(pRmApi,
+                        pGpu->hInternalClient,
+                        pGpu->hInternalSubdevice,
+                        NV2080_CTRL_CMD_INTERNAL_MEMMGR_MEMORY_TRANSFER_WITH_GSP,
+                        &gspParams,
+                        sizeof(gspParams)),
+        failed);
+
+    // Copy from staging buffer to destination
+    if (memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM)
+    {
+        NV_ASSERT_OK_OR_GOTO(status,
+            memdescMapOld(pDst->pMemDesc, 0, size, NV_TRUE,
+                          NV_PROTECT_READ_WRITE, (void**)&pMap, &pPriv),
+            failed);
+
+        portMemCopy(pMap + pDst->offset, size, pStagingBufMap, size);
+
+        memdescUnmapOld(pDst->pMemDesc, NV_TRUE, 0, (void*)pMap, pPriv);
+    }
+
+failed:
+    _memmgrUnmapAndFreeSurface(pStagingBuf, pStagingBufMap, pStagingBufPriv);
+    return status;
+}
+
+static NV_STATUS
+_memmgrMemsetWithGsp
+(
+    OBJGPU           *pGpu,
+    TRANSFER_SURFACE *pDst,
+    NvU32             value,
+    NvU64             size
+)
+{
+    NV2080_CTRL_INTERNAL_MEMMGR_MEMORY_TRANSFER_WITH_GSP_PARAMS gspParams;
+    RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
+
+    // Do not expect to use GSP to memset surfaces in sysmem
+    if (memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM)
+        return NV_ERR_NOT_SUPPORTED;
+
+    portMemSet(&gspParams, 0, sizeof(gspParams));
+
+    gspParams.memop = NV2080_CTRL_MEMMGR_MEMORY_OP_MEMSET;
+    gspParams.transferSize = size;
+    gspParams.value = value;
+    gspParams.dst.baseAddr = memdescGetPhysAddr(pDst->pMemDesc, AT_GPU, 0);
+    gspParams.dst.size = memdescGetSize(pDst->pMemDesc);
+    gspParams.dst.offset = pDst->offset;
+    gspParams.dst.cpuCacheAttrib = memdescGetCpuCacheAttrib(pDst->pMemDesc);
+    gspParams.dst.aperture = memdescGetAddressSpace(pDst->pMemDesc);
+
+    // Send the control call
+    NV_ASSERT_OK_OR_RETURN(
+        pRmApi->Control(pRmApi,
+                        pGpu->hInternalClient,
+                        pGpu->hInternalSubdevice,
+                        NV2080_CTRL_CMD_INTERNAL_MEMMGR_MEMORY_TRANSFER_WITH_GSP,
+                        &gspParams,
+                        sizeof(gspParams)));
+
+    return NV_OK;
 }
 
 /*!
@@ -100,7 +456,16 @@ memmgrMemCopyWithTransferType
             memdescUnmapInternal(pGpu, pDstInfo->pMemDesc, flags);
             break;
         case TRANSFER_TYPE_GSP_DMA:
-            NV_PRINTF(LEVEL_INFO, "Add call to GSP DMA task\n");
+            if (IS_GSP_CLIENT(pGpu))
+            {
+                NV_PRINTF(LEVEL_INFO, "Calling GSP DMA task\n");
+                NV_ASSERT_OK_OR_RETURN(
+                    _memmgrMemcpyWithGsp(pGpu, pDstInfo, pSrcInfo, size));
+            }
+            else
+            {
+                NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_ARGUMENT);
+            }
             break;
         case TRANSFER_TYPE_CE:
             NV_PRINTF(LEVEL_INFO, "Add call to CE\n");
@@ -151,7 +516,16 @@ memmgrMemSetWithTransferType
             memdescUnmapInternal(pGpu, pDstInfo->pMemDesc, flags);
             break;
         case TRANSFER_TYPE_GSP_DMA:
-            NV_PRINTF(LEVEL_INFO, "Add call to GSP DMA task\n");
+            if (IS_GSP_CLIENT(pGpu))
+            {
+                NV_PRINTF(LEVEL_INFO, "Calling GSP DMA task\n");
+                NV_ASSERT_OK_OR_RETURN(
+                    _memmgrMemsetWithGsp(pGpu, pDstInfo, value, size));
+            }
+            else
+            {
+                NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_ARGUMENT);
+            }
             break;
         case TRANSFER_TYPE_CE:
             NV_PRINTF(LEVEL_INFO, "Add call to CE\n");
@@ -179,7 +553,7 @@ memmgrMemWriteMapAndCopy
     void              *pBuf,
     NvU64              offset,
     NvU64              size,
-    NvU32              flags  
+    NvU32              flags
 )
 {
     NvU8   *pDst = NULL;
@@ -231,9 +605,9 @@ memmgrMemWriteInBlocks
         NV_CHECK_OK_OR_RETURN(LEVEL_SILENT, memdescCreateSubMem(&pSubMemDesc, pMemDesc, pGpu, offset + baseOffset, mapSize));
 
         // Set the offset to 0, as the sub descriptor already starts at the offset
-        status = memmgrMemWriteMapAndCopy(pMemoryManager, pSubMemDesc, (NvU8 *)pBuf + offset, 
+        status = memmgrMemWriteMapAndCopy(pMemoryManager, pSubMemDesc, (NvU8 *)pBuf + offset,
                                           0, mapSize, flags);
-                                        
+
         memdescFree(pSubMemDesc);
         memdescDestroy(pSubMemDesc);
 
@@ -266,6 +640,7 @@ memmgrMemWriteWithTransferType
 )
 {
     NvU8 *pMapping = memdescGetKernelMapping(pDstInfo->pMemDesc);
+    OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
 
     // Sanitize the input
     NV_ASSERT_OR_RETURN(pDstInfo != NULL, NV_ERR_INVALID_ARGUMENT);
@@ -291,7 +666,17 @@ memmgrMemWriteWithTransferType
             }
             break;
         case TRANSFER_TYPE_GSP_DMA:
-            NV_PRINTF(LEVEL_INFO, "Add call to GSP DMA task\n");
+            if (IS_GSP_CLIENT(pGpu))
+            {
+                NV_PRINTF(LEVEL_INFO, "Calling GSP DMA task\n");
+                NV_ASSERT_OK_OR_RETURN(
+                    _memmgrMemReadOrWriteWithGsp(pGpu, pDstInfo, pBuf, size,
+                                                 NV_FALSE /* bRead */));
+            }
+            else
+            {
+                NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_ARGUMENT);
+            }
             break;
         case TRANSFER_TYPE_CE:
             NV_PRINTF(LEVEL_INFO, "Add call to CE\n");
@@ -352,7 +737,17 @@ memmgrMemReadWithTransferType
             memdescUnmapInternal(pGpu, pSrcInfo->pMemDesc, 0);
             break;
         case TRANSFER_TYPE_GSP_DMA:
-            NV_PRINTF(LEVEL_INFO, "Add call to GSP DMA task\n");
+            if (IS_GSP_CLIENT(pGpu))
+            {
+                NV_PRINTF(LEVEL_INFO, "Calling GSP DMA task\n");
+                NV_ASSERT_OK_OR_RETURN(
+                    _memmgrMemReadOrWriteWithGsp(pGpu, pSrcInfo, pBuf, size,
+                                                 NV_TRUE /* bRead */));
+            }
+            else
+            {
+                NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_ARGUMENT);
+            }
             break;
         case TRANSFER_TYPE_CE:
             NV_PRINTF(LEVEL_INFO, "Add call to CE\n");
@@ -460,6 +855,31 @@ void memUtilsInitFBAllocInfo
     }
 }
 
+
+MEMORY_DESCRIPTOR *
+memmgrMemUtilsGetMemDescFromHandle_IMPL
+(
+    MemoryManager *pMemoryManager,
+    NvHandle hClient,
+    NvHandle hMemory
+)
+{
+    RsResourceRef *pMemoryRef;
+    Memory        *pMemory;
+
+    if (serverutilGetResourceRef(hClient, hMemory, &pMemoryRef) != NV_OK)
+    {
+        return NULL;
+    }
+
+    pMemory = dynamicCast(pMemoryRef->pResource, Memory);
+    if (pMemory == NULL)
+    {
+        return NULL;
+    }
+    return pMemory->pMemDesc;
+}
+
 /*!
  * @brief This function is used for copying data b/w two memory regions
  *        Both memory regions can be in the same aperture of different apertures
@@ -479,7 +899,8 @@ memmgrMemCopy_IMPL
     NvU32             flags
 )
 {
-    TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager);
+    TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager,
+                                                          pDstInfo, pSrcInfo);
 
     return memmgrMemCopyWithTransferType(pMemoryManager, pDstInfo, pSrcInfo,
                                          size, transferType, flags);
@@ -503,7 +924,8 @@ memmgrMemSet_IMPL
     NvU32             flags
 )
 {
-    TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager);
+    TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager,
+                                                          pDstInfo, NULL);
 
     return memmgrMemSetWithTransferType(pMemoryManager, pDstInfo, value,
                                         size, transferType, flags);
@@ -526,7 +948,8 @@ memmgrMemDescMemSet_IMPL
 )
 {
     TRANSFER_SURFACE transferSurface = {.offset = 0, .pMemDesc = pMemDesc};
-    TRANSFER_TYPE    transferType = memmgrGetMemTransferType(pMemoryManager);
+    TRANSFER_TYPE    transferType = memmgrGetMemTransferType(pMemoryManager,
+                                                             &transferSurface, NULL);
 
     return memmgrMemSetWithTransferType(pMemoryManager, &transferSurface, value,
                                         (NvU32)memdescGetSize(pMemDesc),
@@ -552,7 +975,8 @@ memmgrMemWrite_IMPL
     NvU32             flags
 )
 {
-    TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager);
+    TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager,
+                                                          pDstInfo, NULL);
 
     return memmgrMemWriteWithTransferType(pMemoryManager, pDstInfo, pBuf,
                                           size, transferType, flags);
@@ -577,7 +1001,8 @@ memmgrMemRead_IMPL
     NvU32             flags
 )
 {
-    TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager);
+    TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager,
+                                                          NULL, pSrcInfo);
 
     return memmgrMemReadWithTransferType(pMemoryManager, pSrcInfo, pBuf,
                                          size, transferType, flags);
@@ -599,7 +1024,8 @@ memmgrMemBeginTransfer_IMPL
     NvU32              flags
 )
 {
-    TRANSFER_TYPE      transferType = memmgrGetMemTransferType(pMemoryManager);
+    TRANSFER_TYPE      transferType = memmgrGetMemTransferType(pMemoryManager,
+                                                               pTransferInfo, NULL);
     MEMORY_DESCRIPTOR *pMemDesc     = pTransferInfo->pMemDesc;
     NvU64              offset       = pTransferInfo->offset;
     OBJGPU            *pGpu         = ENG_GET_GPU(pMemoryManager);
@@ -631,12 +1057,12 @@ memmgrMemBeginTransfer_IMPL
 
                 NV_ASSERT_OR_RETURN(memdescMap(pMemDesc, offset, memSz, NV_TRUE, protect,
                     (NvP64*) &pPtr, &pPriv) == NV_OK, NULL);
-                memdescSetKernelMappingPriv(pMemDesc, pPtr);
+                memdescSetKernelMappingPriv(pMemDesc, pPriv);
                 break;
             }
             NV_ASSERT_OR_RETURN((pPtr = memdescMapInternal(pGpu, pMemDesc, flags)) != NULL, NULL);
             pPtr = &pPtr[offset];
-            
+
             break;
         case TRANSFER_TYPE_GSP_DMA:
         case TRANSFER_TYPE_CE:
@@ -672,16 +1098,18 @@ memmgrMemEndTransfer_IMPL
     NvU32              flags
 )
 {
-    TRANSFER_TYPE      transferType = memmgrGetMemTransferType(pMemoryManager);
+    TRANSFER_TYPE      transferType = memmgrGetMemTransferType(pMemoryManager,
+                                                               pTransferInfo, NULL);
     MEMORY_DESCRIPTOR *pMemDesc     = pTransferInfo->pMemDesc;
     NvU64              offset       = pTransferInfo->offset;
     OBJGPU            *pGpu         = ENG_GET_GPU(pMemoryManager);
     NvU64              memSz        = 0;
-    NvU8              *pMapping     = memdescGetKernelMapping(pMemDesc);
+    NvU8              *pMapping     = NULL;
 
     NV_ASSERT_OR_RETURN_VOID(pMemDesc != NULL);
-    NV_ASSERT_OR_RETURN_VOID((memSz = memdescGetSize(pMemDesc)) >= (shadowBufSize + offset) );
+    pMapping = memdescGetKernelMapping(pMemDesc);
 
+    NV_ASSERT_OR_RETURN_VOID((memSz = memdescGetSize(pMemDesc)) >= (shadowBufSize + offset) );
     memSz = shadowBufSize == 0 ? memSz : shadowBufSize;
 
     memdescSetKernelMapping(pMemDesc, NULL);
@@ -693,7 +1121,10 @@ memmgrMemEndTransfer_IMPL
             {
                 NvP64 pPriv = memdescGetKernelMappingPriv(pMemDesc);
                 memdescSetKernelMappingPriv(pMemDesc, NULL);
-                memdescUnmap(pMemDesc, NV_TRUE, 0, pMapping, pPriv);
+                if (pMapping != NULL)
+                {
+                    memdescUnmap(pMemDesc, NV_TRUE, 0, pMapping, pPriv);
+                }
                 return;
             }
             memdescUnmapInternal(pGpu, pMemDesc, flags);
@@ -726,6 +1157,11 @@ memmgrMemDescEndTransfer_IMPL
     NvU32 flags
 )
 {
+    if (pMemDesc == NULL)
+    {
+        return;
+    }
+
     TRANSFER_SURFACE transferSurface = {.offset = 0, .pMemDesc = pMemDesc};
     memmgrMemEndTransfer(pMemoryManager, &transferSurface, memdescGetSize(pMemDesc), flags);
 }
@@ -744,6 +1180,7 @@ memmgrMemDescBeginTransfer_IMPL
     NvU32 flags
 )
 {
+    NV_ASSERT_OR_RETURN(pMemDesc != NULL, NULL);
     TRANSFER_SURFACE transferSurface = {.offset = 0, .pMemDesc = pMemDesc};
     return memmgrMemBeginTransfer(pMemoryManager, &transferSurface, memdescGetSize(pMemDesc), flags);
 }
@@ -910,7 +1347,7 @@ memmgrAllocResources_IMPL
         (addrSpace == ADDR_FBMEM))
     {
         OBJSYS *pSys         = SYS_GET_INSTANCE();
-        NvU32   hostPageSize = pSys->cpuInfo.hostPageSize;
+        NvU64   hostPageSize = pSys->cpuInfo.hostPageSize;
 
         // hostPageSize *should* always be set, but....
         if (hostPageSize == 0)
