@@ -209,6 +209,15 @@ sysmemConstruct_IMPL
 
     memdescSetFlag(pMemDesc, MEMDESC_FLAGS_SYSMEM_OWNED_BY_CLIENT, NV_TRUE);
 
+    if ((sysGetStaticConfig(SYS_GET_INSTANCE()))->bOsSevEnabled &&
+        gpuIsCCorApmFeatureEnabled(pGpu) &&
+        FLD_TEST_DRF(OS32, _ATTR2, _MEMORY_PROTECTION, _UNPROTECTED,
+                     pAllocData->attr2))
+        {
+            memdescSetFlag(pMemDesc, MEMDESC_FLAGS_ALLOC_IN_UNPROTECTED_MEMORY,
+                           NV_TRUE);
+        }
+
     memdescSetGpuCacheAttrib(pMemDesc, gpuCacheAttrib);
 
     rmStatus = memdescAlloc(pMemDesc);
@@ -476,7 +485,7 @@ sysmemAllocResources
     // While replaying a trace, it is possible for the playback OS to have a smaller page size
     // than the capture OS so if we're running a replay where the requested page size is larger,
     // assume this is a contiguous piece of memory, if contiguity is not specified.
-    // 
+    //
     if (FLD_TEST_DRF(OS32, _ATTR, _PHYSICALITY, _DEFAULT, pVidHeapAlloc->attr))
     {
         if ((FLD_TEST_DRF(OS32, _ATTR, _PAGE_SIZE, _BIG, pVidHeapAlloc->attr) ||
